const float4x4 View;
const float4x4 Proj;

#ifndef XBOX360
const float4 Params0[48];
const float4 Params1[48];
const float4 Params2[48];
#endif

const float3 LightColors[64];

struct VSInput
{
    float4 Position : POSITION0;
    float4 Color : COLOR0;
};

struct VSOutputNormal
{
    float4 Position : POSITION0;
    float4 Color : COLOR0;
    float3 Normal : TEXCOORD0;
    float2 TexCoord : TEXCOORD1;
};

struct VSOutput
{
    float4 Position : POSITION0;
    float4 Color : COLOR0;
};

struct PSInput
{
    float4 Color : COLOR0;
    float3 Normal : TEXCOORD0;
    float2 TexCoord : TEXCOORD1;
};

float3 TransformByQuaternion(float3 position, float4 quaternion)
{
    float4 Q = quaternion;
    float3 v = position;
    return
    (2.0f * Q.w * Q.w - 1.0f) * v +
    (2.0f * dot( v, Q.xyz ) * Q.xyz) +
    (2.0f * Q.w * cross(Q.xyz, v));
}

VSOutputNormal TransformLight(VSInput input, float4 param0, float4 param1, float4 param2:COLOR1)
{
    VSOutputNormal output;
    float3 offset = param0.xyz;
    float3 scale = param0.w;
    float3 vtpos = TransformByQuaternion(input.Position * scale, param1);
    float4 pos = float4(offset + vtpos, 1);
    float4 vpos = mul(pos, View);
    output.Position = mul(vpos, Proj);
    output.Color = param2;
    float3 lightColor = 0;
    int lx = pos.x / 25 + 4;
    int ly = pos.y / 25 + 4;
    int li = lx + ly * 8;
    lightColor += LightColors[li - 8] * 0.5;
    lightColor += LightColors[li - 1] * 0.5;
    lightColor += LightColors[li + 1] * 0.5;
    lightColor += LightColors[li + 8] * 0.5;
    lightColor += LightColors[li];
    output.Color.rgb *= 1 - smoothstep(0, 500, length(lightColor));
    output.Color.rgb += lightColor * 0.002;
    output.Normal.xyz = normalize(vtpos.xyz);
    output.TexCoord.x = input.Position.x * 50;
    output.TexCoord.y = (input.Position.y + input.Position.z) * 20;
    return output;
}

static float groundZ = -1;

VSOutputNormal TransformReflect(VSInput input, float4 param0, float4 param1, float4 param2:COLOR1)
{
    VSOutputNormal output;
    float3 offset = param0.xyz;
    offset.z -= (offset.z - groundZ) * 2;
    float3 scale = param0.w;
    float3 vtpos = TransformByQuaternion(input.Position * scale, param1);
    vtpos.z -= (vtpos.z - groundZ) * 2;
    float4 pos = float4(offset + vtpos, 1);
    float4 vpos = mul(pos, View);
    output.Position = mul(vpos, Proj);
    output.Color = param2;
    float3 lightColor = 0;
    int lx = pos.x / 25 + 4;
    int ly = pos.y / 25 + 4;
    int li = lx + ly * 8;
    lightColor += LightColors[li - 8] * 0.5;
    lightColor += LightColors[li - 1] * 0.5;
    lightColor += LightColors[li + 1] * 0.5;
    lightColor += LightColors[li + 8] * 0.5;
    lightColor += LightColors[li];
    output.Color.rgb *= 1 - smoothstep(0, 500, length(lightColor));
    output.Color.rgb += lightColor * 0.002;
    output.Normal.xyz = normalize(vtpos.xyz);
    output.TexCoord.x = input.Position.x * 50;
    output.TexCoord.y = (input.Position.y + input.Position.z) * 20;
    return output;
}

VSOutput Transform(VSInput input, float4 param0, float4 param1, float4 param2:COLOR1)
{
    VSOutput output;
    float3 offset = param0.xyz;
    float3 scale = param0.w;
    float3 vtpos = TransformByQuaternion(input.Position * scale, param1);
    float4 pos = float4(offset + vtpos, 1);
    float4 vpos = mul(pos, View);
    output.Position = mul(vpos, Proj);
    output.Color = param2;
    return output;
}

#ifndef XBOX360
VSOutputNormal TransformLightSI(VSInput input, float index:TEXCOORD0)
{
    return TransformLight(input, Params0[index], Params1[index], Params2[index]);
}

VSOutputNormal TransformReflectSI(VSInput input, float index:TEXCOORD0)
{
    return TransformReflect(input, Params0[index], Params1[index], Params2[index]);
}

VSOutput TransformSI(VSInput input, float index:TEXCOORD0)
{
    return Transform(input, Params0[index], Params1[index], Params2[index]);
}
#else

const int VertexCount;
static float4 param0;
static float4 param1;
static float4 param2;

VSInput CreateVSInput(int index)
{
    int vi = (index + 0.5) % VertexCount;
    int ii = (index + 0.5) / VertexCount;
    float4 pos;
    float4 color;
    asm
    {
        vfetch pos, vi, position0
        vfetch color, vi, color0
        vfetch param0, ii, texcoord0
        vfetch param1, ii, texcoord1
        vfetch param2, ii, color1
    };
    VSInput input;
    input.Position = pos;
    input.Color = color;
    return input;
}

VSOutputNormal TransformLightHI(int index:INDEX)
{
    VSInput input = CreateVSInput(index);
    return TransformLight(input, param0, param1, param2);
}

VSOutputNormal TransformReflectHI(int index:INDEX)
{
    VSInput input = CreateVSInput(index);
    return TransformReflect(input, param0, param1, param2);
}

VSOutput TransformHI(int index:INDEX)
{
    VSInput input = CreateVSInput(index);
    return Transform(input, param0, param1, param2);
}
#endif

static float3 lightVector = {0.6, 0.6, 0.6};

float4 FillLight(PSInput input):COLOR0
{
    float4 color;
    color.rgb = input.Color.rgb;
    color.a = input.Color.a * ((dot(input.Normal, lightVector) + 1) / 2);
    color.a *= (0.75 + sin(input.TexCoord.x + input.TexCoord.y) * 0.25);
    return color;
}

float4 Fill(float4 color:COLOR0):COLOR0
{
    return color;
}

technique LightTech
{
    pass Pass1
    {
#ifdef XBOX360
        VertexShader = compile vs_3_0 TransformLightHI();
#else
        VertexShader = compile vs_2_0 TransformLightSI();
#endif
        PixelShader = compile ps_2_0 FillLight();
    }
}

technique ReflectTech
{
    pass Pass1
    {
#ifdef XBOX360
        VertexShader = compile vs_3_0 TransformReflectHI();
#else
        VertexShader = compile vs_2_0 TransformReflectSI();
#endif
        PixelShader = compile ps_2_0 FillLight();
    }
}

technique BaseTech
{
    pass Pass1
    {
#ifdef XBOX360
        VertexShader = compile vs_3_0 TransformHI();
#else
        VertexShader = compile vs_2_0 TransformSI();
#endif
        PixelShader = compile ps_2_0 Fill();
    }
}

sampler samplerState; 

float4 FillSampler(float2 texCoord:TEXCOORD0):COLOR0
{
    return tex2D(samplerState, texCoord);
}

static float2 offsets[12] =
{
    -0.3, -0.4, -0.8, -0.1, -0.7, 0.5, -0.2, 0.6, 1.0, -0.1, 0.5, -0.5,
    0.5, 0.8, 0.2, -0.9, 0.5, 0.1, 0.9, 0.4, -0.3, -1.0, -0.8, -0.6,
};

float4 FillSamplerBloom(float2 texCoord:TEXCOORD0):COLOR0
{
    float4 sum = tex2D(samplerState, texCoord);
    for (int i = 0; i < 12; i++)
        sum += tex2D(samplerState, texCoord + 0.01 * offsets[i]);
    sum /= 5;
    return sum;
}

technique FillTech
{
    pass Pass1
    {
        PixelShader = compile ps_1_1 FillSampler();
    }
}

technique FillBloomTech
{
    pass Pass1
    {
        PixelShader = compile ps_2_0 FillSamplerBloom();
    }
}
