/*
 * $Id: Frame.cs,v 1.1.1.1 2009/08/22 11:57:01 kenta Exp $
 *
 * Copyright 2009 Kenta Cho. Some rights reserved.
 */
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.GamerServices;
using Abagames.Util;

namespace Abagames.HundredMonkeys
{
    /// <summary>
    /// A game state and effect parameters.
    /// </summary>
    public class Frame : Abagames.Util.Frame
    {
        private const string titleName = "HundredMonkeys";
        private const int versionNum = 10;
        private Random random = new Random();
        private EffectParameter viewParam;
        private EffectParameter projParam;
        private Pad pad;
        private ActorPools actors;
        private int pauseTicks;
        private bool backPressed;
        private bool startPressed;
        private bool fullScreenPressed;
        private RenderTarget2D bloomSurface;
        private int screenWidth, screenHeight;
        private SpriteBatch spriteBatch;

        public override void Initialize()
        {
            base.Initialize();
#if XBOX360
          game.Components.Add(new GamerServicesComponent(game));
#endif
        }

        public override void LoadContent()
        {
            window.Title = titleName;
            effectFileName = @"Shaders\Base";
            defaultTechniqueName = "LightTech";
            base.LoadContent();
            viewParam = Effect.Parameters["View"];
            projParam = Effect.Parameters["Proj"];
            spriteBatch = new SpriteBatch(Gd);
            screenWidth = window.ClientBounds.Width;
            screenHeight = window.ClientBounds.Height;
            ProjMatrix = Matrix.CreatePerspectiveFieldOfView((float)Math.PI / 2.0f, (float)screenWidth / screenHeight, 0.1f, 100000.0f);
            projParam.SetValue(ProjMatrix);
            bloomSurface = CreateRenderTarget(Gd, 1, Gd.PresentationParameters.BackBufferFormat);
            Shape.Initialize(this);
            Letter.Initialize(content, this);
            pad = new Pad();
            actors = new ActorPools(this, content, pad);
            StartGame();
        }

        private RenderTarget2D CreateRenderTarget(GraphicsDevice device, int numberLevels, SurfaceFormat surface)
        {
            if (!GraphicsAdapter.DefaultAdapter.CheckDeviceFormat(DeviceType.Hardware,
                GraphicsAdapter.DefaultAdapter.CurrentDisplayMode.Format, TextureUsage.None,
                QueryUsages.None, ResourceType.RenderTarget, surface))
                surface = device.DisplayMode.Format;
            return new RenderTarget2D(device,
                device.PresentationParameters.BackBufferWidth,
                device.PresentationParameters.BackBufferHeight,
                numberLevels, surface,
                device.PresentationParameters.MultiSampleType,
                device.PresentationParameters.MultiSampleQuality);

        }

        public override void SetRenderState(RenderState renderState) {
            renderState.CullMode = CullMode.None;
            renderState.DepthBufferEnable = true;
            renderState.AlphaBlendEnable = true;
            renderState.SourceBlend = Blend.SourceAlpha;
            renderState.DestinationBlend = Blend.InverseSourceAlpha;
        }

        public void StartGame()
        {
            int randomSeed = random.Next();
            Start(randomSeed);
            pauseTicks = -1;
            startPressed = true;
        }

        public void StartGameOver()
        {
        }

        public void Start(int randomSeed)
        {
            actors.Initialize(randomSeed);
            backPressed = true;
        }

        public override void Update()
        {
            pad.Update();
            bool bp = false;
            if (pad.ButtonBack)
            {
                if (!backPressed)
                    bp = true;
                backPressed = true;
            }
            else
            {
                backPressed = false;
            }
            if (pad.ButtonFullScreen)
            {
                if (!fullScreenPressed)
                    graphics.ToggleFullScreen();
                fullScreenPressed = true;
            }
            else
            {
                fullScreenPressed = false;
            }
            if (bp)
            {
                Exit();
                return;
            }
            if (pad.ButtonStart)
            {
                if (!startPressed)
                {
                    startPressed = true;
                    if (pauseTicks < 0)
                        pauseTicks = 0;
                    else
                        pauseTicks = -1;
                }
            }
            else
            {
                startPressed = false;
            }
            if (pauseTicks >= 0)
                pauseTicks++;
            UpdateInGame();
        }

        private void UpdateInGame()
        {
            if (pauseTicks < 0)
                actors.Update();
        }

        public override void Draw()
        {
            DrawInGame();
        }

        private void DrawInGame()
        {
            SetRenderState(Gd.RenderState);
            Gd.SetRenderTarget(0, bloomSurface);
            Gd.Clear(Color.TransparentBlack);
            actors.Draw();
            ChangeTechnique("BaseTech");
            actors.DrawParticle();
            Gd.SetRenderTarget(0, null);
            ClearScreen();
            ChangeTechnique("ReflectTech");
            actors.Draw();
            actors.DrawParticle();
            ChangeTechnique("LightTech");
            actors.Draw();
            ChangeTechnique("BaseTech");
            actors.DrawParticle();
            Gd.RenderState.DepthBufferEnable = false;
            ChangeTechnique("LightTech");
            actors.DrawPanel();
            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Immediate, SaveStateMode.None);
            ChangeTechnique("FillBloomTech");
            spriteBatch.Draw(bloomSurface.GetTexture(), Vector2.Zero, Color.White);
            spriteBatch.End();
            SetRenderState(Gd.RenderState);
            Gd.RenderState.DepthBufferEnable = false;
            ChangeTechnique("BaseTech");
            actors.DrawGameState();
            if (pauseTicks >= 0 && pauseTicks % 60 < 30)
                Letter.Add("PAUSE", new Vector3(-3, -4, 0), 0.7f, Quaternion.Identity, 1);
            Letter.Draw();
        }

        private void ClearScreen()
        {
            Gd.Clear(Color.Black);
        }

        public void LookAt(Vector3 from, Vector3 to, Vector3 up)
        {
            ViewMatrix = Matrix.CreateLookAt(from, to, up);
            viewParam.SetValue(ViewMatrix);
        }

        public int PauseTicks
        {
            get
            {
                return pauseTicks;
            }
        }
    }
}
