/*
 * $Id: Record.cs,v 1.1.1.1 2009/08/22 11:57:02 kenta Exp $
 *
 * Copyright 2009 Kenta Cho. Some rights reserved.
 */
using System;
using System.IO;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.GamerServices;

namespace Abagames.Util
{
    /// <summary>
    /// Record a score ranking.
    /// </summary>
    public class Record
    {
        public const int RankingNum = 10;
        private int versionNum;
        private string fileName;
        private string titleName;
        private int[] scores;
        private int lastScore;
        private int lastRank;
        private string path;
        private StorageDevice device;
        private bool isLoadingRequested;
        private bool isSavingRequested;
        private IAsyncResult result;
        private int baseScore;

        public Record(String fileName, String titleName, int versionNum, int baseScore)
        {
            this.fileName = fileName;
            this.titleName = titleName;
            this.versionNum = versionNum;
            this.baseScore = baseScore;
            scores = new int[RankingNum];
            isLoadingRequested = isSavingRequested = false;
            device = null;
            result = null;
        }

        public void Update()
        {
            if (result != null && result.IsCompleted)
            {
                device = Guide.EndShowStorageDeviceSelector(result);
                if (device != null && !device.IsConnected)
                    device = null;
                result = null;
            }
            if (isLoadingRequested && device != null)
            {
                StorageContainer container = device.OpenContainer(titleName);
                path = Path.Combine(container.Path, fileName);
                LoadRecord();
                container.Dispose();
                isLoadingRequested = false;
            }
            if (isSavingRequested && device != null)
            {
                StorageContainer container = device.OpenContainer(titleName);
                path = Path.Combine(container.Path, fileName);
                SaveRecord();
                container.Dispose();
                isSavingRequested = false;
            }
        }

        public void Load()
        {
            isLoadingRequested = true;
            if (device == null)
                result = Guide.BeginShowStorageDeviceSelector(PlayerIndex.One, null, null);
        }

        public void Save()
        {
            isSavingRequested = true;
            if (device == null)
                result = Guide.BeginShowStorageDeviceSelector(PlayerIndex.One, null, null);
        }

        private void LoadRecord()
        {
            Initialize();
            BinaryReader br = null;
            try
            {
                br = new BinaryReader(File.Open(path, FileMode.Open, FileAccess.Read));
                int vn = br.ReadInt32();
                if (vn != versionNum)
                    throw new IOException("Invalid version num");
                for (int i = 0; i < RankingNum; i++)
                    scores[i] = br.ReadInt32();
            }
            catch
            {
            }
            finally
            {
                if (br != null)
                    br.Close();
            }
        }

        private void Initialize()
        {
            for (int i = 0; i < RankingNum; i++)
                scores[i] = (RankingNum - i) * baseScore;
            lastScore = 0;
            lastRank = -1;
        }

        private void SaveRecord()
        {
            BinaryWriter bw = null;
            try
            {
                bw = new BinaryWriter(File.Open(path, FileMode.Create, FileAccess.Write));
                bw.Write(versionNum);
                for (int i = 0; i < RankingNum; i++)
                    bw.Write(scores[i]);
            }
            catch
            {
            }
            finally
            {
                if (bw != null)
                    bw.Close();
            }
        }

        public void RecordScore(int s)
        {
            lastScore = s;
            lastRank = -1;
            for (int i = 0; i < RankingNum; i++)
            {
                if (s > scores[i])
                {
                    for (int j = RankingNum - 2; j >= i; j--)
                        scores[j + 1] = scores[j];
                    lastRank = i;
                    scores[i] = s;
                    break;
                }
            }
            Save();
        }

        public int[] Scores
        {
            get
            {
                return scores;
            }
        }

        public int LastScore
        {
            get
            {
                return lastScore;
            }
        }

        public int LastRank
        {
            get
            {
                return lastRank;
            }
        }
    }
}
