/*
 * $Id: Expression.cs,v 1.1.1.1 2009/08/22 11:57:02 kenta Exp $
 *
 * Copyright 2008 Kenta Cho. Some rights reserved.
 */
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;

namespace Abagames.Util.Ssl
{
    /// <summary>
    /// 
    /// </summary>
    public class Expression
    {
        public struct Operator
        {
            public enum Type
            {
                Plus, Minus, Multiple, Division, Modulo, Variable, Number, Infinite, Random,
            };
            public Type OperatorType;
            public int Index;
        }
        private static Random random = new Random();
        public Stack<Operator> Operators;
        public List<float> Numbers;
        private String text;

        public Expression()
        {
        }

        public Expression(string t)
        {
            text = t;
            Operators = new Stack<Operator>();
            Numbers = new List<float>();
            ParseToRpn(0, text.Length);
        }

        public unsafe float Calc(float* parameters)
        {
            Stack<float> stack = new Stack<float>();
            foreach (Operator op in Operators)
            {
                switch (op.OperatorType)
                {
                case Operator.Type.Number:
                    stack.Push(Numbers[op.Index]);
                    break;
                case Operator.Type.Variable:
                    stack.Push(parameters[op.Index]);
                    break;
                case Operator.Type.Infinite:
                    stack.Push(float.PositiveInfinity);
                    break;
                case Operator.Type.Random:
                    stack.Push((float)random.NextDouble());
                    break;
                default:
                    float n1 = stack.Pop();
                    float n2 = stack.Pop();
                    stack.Push(CalcOperator(op.OperatorType, n2, n1));
                    break;
                }
            }
            return stack.Pop();
        }

        private void ParseToRpn(int from, int to)
        {
            int op0 = -1;
            int op1 = -1;
            for (int i = to - 1; i >= from; i--)
            {
                char c = text[i];
                if (c == ')')
                {
                    int bc = 1;
                    do
                    {
                        i--;
                        if (text[i] == ')')
                            bc++;
                        else if (text[i] == '(')
                            bc--;
                    } while (bc > 0);
                }
                else if (op0 < 0 && (c == '*' || c == '/' || c =='%'))
                {
                    op0 = i;
                }
                else if (c == '+' || c =='-')
                {
                    op1 = i;
                    break;
                }
            }
            if (op0 < 0 && op1 < 0 && text[from] == '(' && text[to - 1] == ')')
            {
                ParseToRpn(from + 1, to - 1);
                return;
            }
            if (op1 < 0)
            {
                if (op0 < 0)
                {
                    ParseFloatValue(from, to - from);
                }
                else
                {
                    ParseToRpn(from, op0);
                    ParseToRpn(op0 + 1, to);
                    switch (text[op0])
                    {
                        case '*':
                            PushOperator(Operator.Type.Multiple);
                            break;
                        case '/':
                            PushOperator(Operator.Type.Division);
                            break;
                        case '%':
                            PushOperator(Operator.Type.Modulo);
                            break;
                        default:
                            throw new FormatException("unknown operator: " + text[op0]);
                    }
                }
            }
            else
            {
                if (op1 == from)
                {
                    switch (text[op1])
                    {
                        case '-':
                            ParseToRpn(from + 1, to);
                            PushNumber(-1);
                            PushOperator(Operator.Type.Multiple);
                            break;
                        case '+':
                            ParseToRpn(from + 1, to);
                            break;
                        default:
                            throw new FormatException("unknown unary operator: " + text[op1]);
                    }
                }
                else
                {
                    ParseToRpn(from, op1);
                    ParseToRpn(op1 + 1, to);
                    switch (text[op1])
                    {
                        case '+':
                            PushOperator(Operator.Type.Plus);
                            break;
                        case '-':
                            PushOperator(Operator.Type.Minus);
                            break;
                        default:
                            throw new FormatException("unknown operator: " + text[op1]);
                    }
                }
            }
        }

        private void ParseFloatValue(int from, int length)
        {
            string v = text.Substring(from, length);
            try
            {
                if (text[from] == '$')
                {
                    PushVariable(int.Parse(text.Substring(from + 1, length - 1)) - 1);
                    return;
                }
                if (v == "inf")
                    PushInfinite();
                else if (v == "rnd")
                    PushRandom();
                else
                    PushNumber(float.Parse(v));
            }
            catch (FormatException e)
            {
                throw new FormatException("Invalid value: " + v, e);
            }
        }

        private void PushOperator(Operator.Type ot)
        {
            Operator i1 = Operators.Pop();
            Operator i2 = Operators.Pop();
            if (i1.OperatorType == Operator.Type.Number && i2.OperatorType == Operator.Type.Number)
            {
                float n1 = Numbers[i1.Index];
                float n2 = Numbers[i2.Index];
                if (i1.Index > i2.Index)
                {
                    Numbers.RemoveAt(i1.Index);
                    Numbers.RemoveAt(i2.Index);
                }
                else
                {
                    Numbers.RemoveAt(i2.Index);
                    Numbers.RemoveAt(i1.Index);
                }
                PushNumber(CalcOperator(ot, n2, n1));
            }
            else
            {
                Operators.Push(i2);
                Operators.Push(i1);
                Operator op = new Operator();
                op.OperatorType = ot;
                Operators.Push(op);
            }
        }

        private void PushVariable(int vi)
        {
            Operator op;
            op.OperatorType = Operator.Type.Variable;
            op.Index = vi;
            Operators.Push(op);
        }

        private void PushNumber(float n)
        {
            Numbers.Add(n);
            Operator op;
            op.OperatorType = Operator.Type.Number;
            op.Index = Numbers.Count - 1;
            Operators.Push(op);
        }

        private void PushInfinite()
        {
            Operator op;
            op.OperatorType = Operator.Type.Infinite;
            op.Index = -1;
            Operators.Push(op);
        }

        private void PushRandom()
        {
            Operator op;
            op.OperatorType = Operator.Type.Random;
            op.Index = -1;
            Operators.Push(op);
        }

        private float CalcOperator(Operator.Type ot, float n1, float n2)
        {
            switch (ot)
            {
            case Operator.Type.Plus:
                return n1 + n2;
            case Operator.Type.Minus:
                return n1 - n2;
            case Operator.Type.Multiple:
                return n1 * n2;
            case Operator.Type.Division:
                return n1 / n2;
            case Operator.Type.Modulo:
                return n1 % n2;
            default:
                throw new FormatException("illegal operator: " + n1 + " " + ot.ToString() + " " + n2);
            }
        }
    }

    public class ExpressionReader : ContentTypeReader<Expression>
    {
        protected override Expression Read(ContentReader input, Expression existingInstance)
        {
            Expression expression = new Expression();
            expression.Operators = new Stack<Expression.Operator>(input.ReadObject<Expression.Operator[]>());
            expression.Numbers = input.ReadObject<List<float>>();
            return expression;
        }
    }

    public class OperatorReader : ContentTypeReader<Expression.Operator>
    {
        protected override Expression.Operator Read(ContentReader input, Expression.Operator existingInstance)
        {
            Expression.Operator op = new Expression.Operator();
            op.OperatorType = input.ReadObject<Expression.Operator.Type>();
            op.Index = input.ReadObject<int>();
            return op;
        }
    }
}
